<?php

namespace Drupal\block_inactive_users\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Class SettingsForm.
 *
 * @package Drupal\block_inactive_users\Form
 */
class SettingsForm extends ConfigFormBase {

  protected $loggerChannelFactory;
  protected $entityTypeManager;

  /**
   * Create function for dependency injection.
   */
  public static function create(ContainerInterface $container) {

    $loggerFactory = $container->get('logger.factory');

    return new static($loggerFactory,
      $container->get('entity_type.manager'));
  }

  /**
   * Constructor.
   */
  public function __construct(LoggerChannelFactoryInterface $loggerChannelFactory, EntityTypeManagerInterface $entityTypeManager) {
    $this->entityTypeManager = $entityTypeManager;
    $this->loggerChannelFactory = $loggerChannelFactory;

  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return "block_inactive_users_settings";
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['block_inactive_users.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('block_inactive_users.settings');
    $roles = user_role_names();
    unset($roles['anonymous']);
    unset($roles['administrator']);

    $form["users_settings"] = [
      "#type" => "details",
      "#title" => $this->t("Inactive Users Management settings"),
      '#open' => TRUE,
    ];
    $form["email_settings"] = [
      "#type" => "details",
      "#title" => $this->t("Email blocked user"),
      '#open' => TRUE,
    ];

    $form['users_settings']['block_inactive_users_idle_time'] = [
      '#title' => $this->t('Time period (in months)'),
      '#required' => TRUE,
      '#type' => 'number',
      '#attributes' => [
        'min' => 0,
      ],
      '#default_value' => $config->get('block_inactive_users.block_inactive_users_idle_time'),
      '#description' => $this->t('Disable inactive users.'),
    ];

    $form['users_settings']['block_inactive_users_exclude_roles'] = [
      '#type' => 'select',
      '#multiple' => TRUE,
      '#title' => $this->t('Exclude users with role(s):'),
      '#default_value' => $config->get('block_inactive_users.block_inactive_users_exclude_roles'),
      '#options' => $roles,
      '#description' => $this->t('Select role(s) to avoid disabling account.'),
    ];

    $form['users_settings']['block_inactive_users_include_never_accessed'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Include users who have never logged in'),
      '#default_value' => $config->get('block_inactive_users.block_inactive_users_include_never_accessed'),
      '#description' => $this->t('If checked, will block users whose last access is set to "never."'),
    ];

    $form['email_settings']['block_inactive_users_from_email'] = [
      '#title' => $this->t('From'),
      '#required' => TRUE,
      '#type' => 'textfield',
      '#default_value' => $config->get('block_inactive_users.block_inactive_users_from_email'),
      '#description' => $this->t('Email "from" email address configuration.'),
    ];

    $form['email_settings']['block_inactive_users_email_subject'] = [
      '#title' => $this->t('Email subject'),
      '#type' => 'textfield',
      '#default_value' => $config->get('block_inactive_users.block_inactive_users_email_subject'),
      '#description' => $this->t('Email subject text.'),
    ];

    $form['email_settings']['block_inactive_users_email_content'] = [
      '#title' => $this->t('Email template'),
      '#type' => 'textarea',
      '#default_value' => $config->get('block_inactive_users.block_inactive_users_email_content'),
      '#description' => $this->t('Provide a template of email to be sent to disabled users. <br/>Replacements values available:  [account-name],  [site-name], [activation-link].'),
    ];

    $form['actions']['block_inactive_users.block_inactive_users_update'] = [
      '#type' => 'submit',
      '#value' => $this->t('Disable inactive users'),
      '#submit' => ['::updateUsers'],
    ];

    return parent::buildForm($form, $form_state);

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $this->configFactory->getEditable('block_inactive_users.settings')
      ->set('block_inactive_users.block_inactive_users_idle_time', $form_state->getValue('block_inactive_users_idle_time'))
      ->set('block_inactive_users.block_inactive_users_from_email', $form_state->getValue('block_inactive_users_from_email'))
      ->set('block_inactive_users.block_inactive_users_email_content', $form_state->getValue('block_inactive_users_email_content'))
      ->set('block_inactive_users.block_inactive_users_email_subject', $form_state->getValue('block_inactive_users_email_subject'))
      ->set('block_inactive_users.block_inactive_users_exclude_roles', $form_state->getValue('block_inactive_users_exclude_roles'))
      ->set('block_inactive_users.block_inactive_users_include_never_accessed', $form_state->getValue('block_inactive_users_include_never_accessed'))
      ->save();

    parent::submitForm($form, $form_state);

  }

  /**
   * Update users status.
   *
   * Disable users after they have idle for time period.
   */
  public function updateUsers(array &$form, FormStateInterface $form_state) {
    $this->submitForm($form, $form_state);
    // Disable inactive users based on configured rules.
    block_inactive_users_block_users();
  }

}
