<?php

namespace Drupal\block_inactive_users;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\user\Entity\User;
use Drupal\user\UserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Mail\Plugin\Mail\PhpMail;
use Drupal\Core\Url;

/**
 * Class InactiveUsersHandler.
 *
 * @package Drupal\block_inactive_users
 */
class InactiveUsersHandler {
  const FORM_SETTINGS_CONFIG_OBJ_NAME = "block_inactive_users.settings";
  const LOGGER_CHANNEL = "block_inactive_users";

  protected $configFactoryService;
  protected $logger;

  /**
   * Constructor.
   */
  public function __construct(ConfigFactoryInterface $config_factory,
                              LoggerChannelFactoryInterface $loggerChannelFactory) {

    $this->configFactoryService = $config_factory->get(self::FORM_SETTINGS_CONFIG_OBJ_NAME);
    $this->logger = $loggerChannelFactory;

  }

  /**
   * Create function for depdendency injection.
   */
  public static function create(ContainerInterface $container) {

    $config_factory = $container->get('config.factory');
    $loggerFactory = $container->get('logger.factory');

    return new static($config_factory, $loggerFactory);

  }

  /**
   * Disables an inactive user account.
   *
   * @param \Drupal\user\Entity\User $user
   *   A user object.
   * @param bool $sendmail
   *   Boolean to trigger email.
   *
   * @return \Drupal\user\Entity\User
   *   Returns a blocked user object.
   */
  public function disableInactiveUsersStatus(User $user, $sendmail = TRUE) {

    $user->block();
    $user->save();
    $this->logger->get(self::LOGGER_CHANNEL)->info($user->getEmail() . ' has been disabled.');

    if ($sendmail) {

      $activation_link = $this->createReactivationUrl($user);

      // Sends email notification.
      $this->mailUser($this->configFactoryService->get('block_inactive_users.block_inactive_users_from_email'),
        $user->getEmail(), $user->getAccountName(),
        $this->configFactoryService->get('block_inactive_users.block_inactive_users_email_subject'),
        $this->configFactoryService->get('block_inactive_users.block_inactive_users_email_content'),
        $activation_link);
    }

    return $user;

  }

  /**
   * Takes 2 timestamps and returns the month interval as a string.
   *
   * @param string $last_access
   *   A timestamp of the last time a user accessed their account.
   * @param string $current_time
   *   time(), Value of the current timestamp.
   *
   * @return int
   *   Returns the interval of the times in months.
   */
  public function timestampdiff($last_access, $current_time) {

    $begin = new \DateTime("@$last_access");
    $end = new \DateTime("@$current_time");
    $months = -1;

    while ($begin <= $end) {

      $begin->modify('first day of next month');
      $months++;

    }

    return $months;

  }

  /**
   * Sends a email notification.
   *
   * @param string $from
   *   From email address.
   * @param string $to
   *   To email address.
   * @param string $user_name
   *   Users first name.
   * @param string $message_subject
   *   Message subject field.
   * @param string $message_content
   *   Email template.
   * @param string $activation_link
   *   Url to re-activate user.
   *
   * @return bool
   *   Returns true if mail was sent successfully.
   */
  public function mailUser($from, $to, $user_name, $message_subject, $message_content, $activation_link) {

    $message = [];

    $system_site_config = \Drupal::config('system.site');
    $site_name = $system_site_config->get('name');
    $send_mail = new PhpMail();

    $message['headers'] = [
      'content-type' => 'text/html',
      'MIME-Version' => '1.0',
      'reply-to' => $from,
      'from' => $site_name . ' <' . $from . '>',
    ];

    $message['to'] = $to;

    $tring_to_replace = [
      '[account-name]' => $user_name,
      '[site-name]' => $site_name,
      '[activation-link]' => $activation_link,

    ];

    $message['subject'] = $message_subject;

    foreach ($tring_to_replace as $key => $value) {
      $message_content = str_replace($key, $value, $message_content);
    }
    $message['body'] = $message_content;

    $result = $send_mail->mail($message);
    if ($result != TRUE) {
      $message = t('There was a problem sending your email notification to @email.', ['@email' => $to]);

      \Drupal::logger('mail-log')->error($message);
      return FALSE;
    }
    $message = t('An email notification has been sent to @email.', ['@email' => $to]);
    \Drupal::logger('mail-log')->notice($message);

    return FALSE;

  }

  /**
   * Generates a URL to confirm an account reactivation request.
   *
   * @param \Drupal\user\UserInterface $account
   *   The user account object.
   * @param array $options
   *   (optional) A keyed array of settings. Supported options are:
   *   - langcode: A language code to be used when generating locale-sensitive
   *     URLs. If langcode is NULL the users preferred language is used.
   *
   * @return string
   *   A unique URL that may be used to confirm the reactivation of the user
   *   account.
   */
  private function createReactivationUrl(UserInterface $account, array $options = []) {

    $timestamp = REQUEST_TIME;
    $langcode = isset($options['langcode']) ? $options['langcode'] : $account->getPreferredLangcode();
    $url_options = ['absolute' => TRUE, 'language' => \Drupal::languageManager()->getLanguage($langcode)];
    return Url::fromRoute('block_inactive_users.reactivate_confirm', [
      'user' => $account->id(),
      'timestamp' => $timestamp,
      'hashed_pass' => user_pass_rehash($account, $timestamp),
    ], $url_options)->toString();

  }

}
